package org.megaJoltLiteJr;

import java.io.*;
import java.util.*;

/** Reads an Ignition Configuration from a File
 * The file is in a standard java properties file format:
 * 
 * #number of cylinders
 * cylinders=4
 *
 * #RPMS are X100
 * rpmBins=10,20,25,30,40,50,60,70,80,90
 * 
 * #MAP are in raw KPa
 * mapBins=10,20,30,40,50,60,70,80,90,100
 *
 * #ignition advance values, MAP row x RPMs
 * advance0=15,20,23,25,30,32,33,35,38,40
 * advance1=15,20,23,25,30,32,33,35,38,40
 * advance2=15,20,23,25,30,32,33,35,38,40
 * advance3=15,20,23,25,30,32,33,35,38,40
 * advance4=15,20,23,25,30,32,33,35,38,40
 * advance5=15,20,23,25,30,32,33,35,38,40
 * advance6=15,20,23,25,30,32,33,35,38,40
 * advance7=15,20,23,25,30,32,33,35,38,40
 * advance8=15,20,23,25,30,32,33,35,38,40
 * advance9=15,20,23,25,30,32,33,35,38,40
 *
 * #user outputs
 * #MAP,HIGH mode,125 KPa
 * userOutType0=0
 * userOutMode0=0
 * userOutValue0=125
 *
 * #RPM,LOW mode, 4000 RPM 
 * userOutType1=1
 * userOutMode1=1
 * userOutValue1=40
 * 
 * #RPM,HIGH mode, 3000 RPM
 * userOutType2=1
 * userOutMode2=0
 * userOutValue2=30
 *
 * #MAP,HIGH mode, 180 KPa
 * userOutType3=0
 * userOutMode3=0
 * userOutValue3=180
 *
 * #Shift light 
 * shiftLight=75
 *
 * #Rev Limit
 * revLimit=80
 *
 * @author  brent
 */

public class IgnitionConfigFileIO {
    
    public static final String CYLINDERS_KEY="cylinders";
    public static final String RPM_BINS_KEY="rpmBins";
    public static final String MAP_BINS_KEY="mapBins";
    public static final String ADVANCE_KEY="advance";
    public static final String USER_OUT_TYPE_KEY="userOutType";
    public static final String USER_OUT_MODE_KEY="userOutMode";
    public static final String USER_OUT_VALUE_KEY="userOutValue";
    public static final String SHIFT_LIGHT_KEY="shiftLight";
    public static final String REV_LIMIT_KEY="revLimit";
    
    public static final String FILE_HEADER="Ignition Configuration for MegaJoltLite Jr.";
    

    
    public void writeIgnitionConfig(File f, IgnitionConfig ic) throws IOException,FileNotFoundException{
     
        if (null==ic) throw new IllegalStateException("Ignition Config not specified");
        if (null==f) throw new IllegalStateException("File not specified");
        
        Properties configProps = new Properties();

        //set cylinders
        configProps.setProperty(CYLINDERS_KEY, String.valueOf(ic.getCylinderCount()));
        
        //write RPM bins
        StringBuffer rpmBins=new StringBuffer();
        for (int i=0;i<IgnitionConfig.RPM_BIN_SIZE;i++){
            rpmBins.append(ic.getRpmBinValue(i));
            if (i<IgnitionConfig.RPM_BIN_SIZE-1) rpmBins.append(",");
        }
        configProps.setProperty(RPM_BINS_KEY, rpmBins.toString());
        
        //write MAP bins
        StringBuffer mapBins=new StringBuffer();
        for (int i=0;i<IgnitionConfig.MAP_BIN_SIZE;i++){
            mapBins.append(ic.getMapBinValue(i));
            if (i<IgnitionConfig.MAP_BIN_SIZE-1) mapBins.append(",");
        }
        configProps.setProperty(MAP_BINS_KEY, mapBins.toString());
        
        //write ignition advance values
        for (int i=0;i<IgnitionConfig.MAP_BIN_SIZE;i++){
            StringBuffer advanceValues = new StringBuffer();
            for (int j=0;j<IgnitionConfig.RPM_BIN_SIZE;j++){
                advanceValues.append(ic.getIgnitionMapValue(j,i));
                if (j<IgnitionConfig.RPM_BIN_SIZE-1) advanceValues.append(",");
            }
            configProps.setProperty(ADVANCE_KEY+String.valueOf(i), advanceValues.toString());
        }
        
        //write user defined outputs
        for (int i=0;i<IgnitionConfig.USER_OUTPUT_SIZE;i++){
            configProps.setProperty(USER_OUT_TYPE_KEY+String.valueOf(i), String.valueOf(ic.getUserOutputType(i)));
            configProps.setProperty(USER_OUT_MODE_KEY+String.valueOf(i), String.valueOf(ic.getUserOutputMode(i)));
            configProps.setProperty(USER_OUT_VALUE_KEY+String.valueOf(i), String.valueOf(ic.getUserOutputValue(i)));
        }
        
        //write shift light value
        configProps.setProperty(SHIFT_LIGHT_KEY, String.valueOf(ic.getShiftLightRpm()));
        
        //write rev limit value
        configProps.setProperty(REV_LIMIT_KEY, String.valueOf(ic.getRevLimitRpm()));
        
        FileOutputStream os = new FileOutputStream(f);
        configProps.store(os, FILE_HEADER);
    }
    
    
    public IgnitionConfig readIgnitionConfig(File f) throws FileNotFoundException,IOException{

        if (null==f) throw new IllegalStateException("Ignition Config File not defined");
        
        InputStream is=new FileInputStream(f);
        if (null==is) throw new IllegalStateException("Could not create input stream for File: " + f.toString());
        
        Properties configProps = new Properties();
        
        try{
            configProps.load(is);
        }
        
        catch (IOException e){
            throw new IOException("Could not read Ignition Config for file: " + f.toString() + "; " + e.getMessage());
        }
            
        IgnitionConfig ic = new IgnitionConfig();

        //read in number of cylinders
        String cylinders=configProps.getProperty(CYLINDERS_KEY);
        ic.setCylinderCount(Integer.valueOf(cylinders).intValue());

        //read in RPM Bins
        String rpmBins=configProps.getProperty(RPM_BINS_KEY);
        String rpmBinValues[]=rpmBins.split(",");
        if (rpmBinValues.length!=IgnitionConfig.RPM_BIN_SIZE) throw new IllegalStateException("RPM Bins read from config file are incorrect (wrong number of values): " + rpmBins);
        for (int i=0;i<IgnitionConfig.RPM_BIN_SIZE;i++){
            ic.setRpmBinValue(i, Integer.valueOf(rpmBinValues[i]).intValue());
        }

        //read in MAP Bins
        String mapBins=configProps.getProperty(MAP_BINS_KEY);
        String mapBinValues[]=mapBins.split(",");
        if (mapBinValues.length!=IgnitionConfig.MAP_BIN_SIZE) throw new IllegalStateException("MAP Bins read from config file are incorrect (wrong number of values): " + mapBins);
        for (int i=0;i<IgnitionConfig.MAP_BIN_SIZE;i++){
            ic.setMapBinValue(i,Integer.valueOf(mapBinValues[i]).intValue());
        }

        //read in ignition advance values
        for (int i=0;i<IgnitionConfig.MAP_BIN_SIZE;i++){
            String advances=configProps.getProperty(ADVANCE_KEY+String.valueOf(i));
            String advanceValues[]=advances.split(",");
            if (advanceValues.length!=IgnitionConfig.RPM_BIN_SIZE) throw new IllegalStateException("Ignition Advance Values are incorrect (wrong number of values): " + advances);
            for (int j=0;j<IgnitionConfig.RPM_BIN_SIZE;j++){
                ic.setIgnitionMapValue(j, i, Integer.valueOf(advanceValues[j]).intValue());
            }
        }

        //read in user defined outputs
        for (int i=0;i<IgnitionConfig.USER_OUTPUT_SIZE;i++){
            String userOutType = configProps.getProperty(USER_OUT_TYPE_KEY+String.valueOf(i));
            ic.setUserOutputType(i, Integer.valueOf(userOutType).intValue());

            String userOutMode = configProps.getProperty(USER_OUT_MODE_KEY+String.valueOf(i));
            ic.setUserOutputMode(i,Integer.valueOf(userOutMode).intValue());

            String userOutValue = configProps.getProperty(USER_OUT_VALUE_KEY+String.valueOf(i));
            ic.setUserOutputValue(i,Integer.valueOf(userOutValue).intValue());
        }

        //read in Shift Light value
        String shiftLight = configProps.getProperty(SHIFT_LIGHT_KEY);
        ic.setShiftLightRpm(Integer.valueOf(shiftLight).intValue());

        //read in Rev Limit value
        String revLimit = configProps.getProperty(REV_LIMIT_KEY);
        ic.setRevLimitRpm(Integer.valueOf(revLimit).intValue());
            
        return ic;
    }

    
    
}
